<?php
/* vim: set et ts=4 sw=4 sts=4 ai: */
// +----------------------------------------------------------------------+
// | Custom/site-specific code tweaks                                     |
// +----------------------------------------------------------------------+
// | @version $Revision: 1.10 $
// +----------------------------------------------------------------------+
//
// This file contains customized/site-specific code tweaks for various
// routines used by ochiba. They all start with "custom_" and will be
// called by ochiba in place of normal processing

/*
 * This takes a comment as pulled from the database and formats it for
 * printing. Note that by default, nl2br(htmlspecialchars(stripslashes()))
 * is called for the comment; if you use this function, you have to do
 * that yourself!
 *
 * Pay careful attention to the order you do things! If you are going to be
 * adding tags, you pretty much need to do the htmlspecialchars first.
 */
function custom_comment_format($string)
{
    $string = stripslashes($string);

    // Needing to do htmlspecialchars() later, use these as placeholders
    // for "<" and ">" to survive it, then translate back subsequently
    $lt = chr(1);
    $gt = chr(2);
    $q = chr(3);
    $k = chr(4);

    // This finds keyword references in the form ">>>keyword" and makes a link
    $string = ereg_replace(">>>([A-z0-9]+)","${lt}a href=${q}" . ROOT . "/keyword/\\1${q}${gt}${k}\\1${lt}/a${gt}",$string);

    // This finds thread/comment references in the form ">>123" and makes a link
    // out of them
    $string = ereg_replace(">>([0-9]+)","${lt}a href=${q}javascript:hilite('\\1')$q${gt}>>\\1${lt}/a${gt}",$string);

    // This finds quotes ("> blah blah blah") and colors them (see also css)
    $string = ereg_replace("(^|[\n\r]+)>([^\n\r>]+)([\n\r])","\\1${lt}cite${gt}>\\2${lt}/cite${gt}\\3",$string);

    // This finds http URLs in the form <http://site/blah/blah>
    $string = ereg_replace("<(http:[^>\r\n]+)>","<${lt}a href=${q}\\1${q}${gt}\\1${lt}/a${gt}>",$string);

    $string = htmlspecialchars($string);

    // Convert the special chars back to "<" and ">"
    $string = strtr($string,array(
        $lt => "<",
        $gt => ">",
        $q => '"',
        $k => '&gt;&gt;&gt;',
    ));
    
    $string = nl2br($string);
    return $string;
}
?>
