<?php
/* vim: set et ts=4 sw=4 sts=4 ai: */
// +----------------------------------------------------------------------+
// | Database (and other) functions                                       |
// +----------------------------------------------------------------------+
// | @version $Revision: 1.42 $
// +----------------------------------------------------------------------+

require_once("DB.php");
// Honor DB as defined by older ochiba installation (file path, rather than
// PEAR:DB DSN)
if(!strstr(DB,'://')) {
    $db = DB::connect("sqlite:///" . DB);
} else {
    $db = DB::connect(DB);
}
if(DB::isError($db)) {
    die("Error");
} else {
    $db->setFetchMode(DB_FETCHMODE_ASSOC);
}

/**
* Fetch a list of categories. By default, will return all categories.
* Unless either $which or $id is provided ($which = abbr, $id = id)
*
* @param	string	$which
*		If not false, get record for category abbr='$which'
*
* @param	int		$id
*		If not false, get record for category id='$id'
*
* @return	mixed
*		Indexed array of associative arrays (column=>value)
*/
function getCategories($which=false,$id=false) {
	global $db;
	$sql = "select * from categories";
	if($which) {
		$sql.=" where abbr='$which'";
	} else if ($id) {
		$sql.=" where id='$id'";
	}
	$sql.= " order by category";
	$result = $db->getAll($sql);
    return DB::isError($result) ? false : $result;
}

function getBans($full=false) {
    global $db;
    $sql = $full ?  "select * from bans" : "select ip from bans";
    if($full) {
        $sql = "select * from bans order by ip";
        $result = $db->getAll($sql);
    } else {
        $sql = "select ip from bans order by ip";
        $result = $db->getCol($sql);
    }
    return DB::isError($result) ? false : $result;
}

/**
* Remove a ban given an IP
*
* @param string $ip The ip address entry (may be a glob)
* @return boolean Status
*/
function removeBan($ip) {
    global $db;
    $sql = "delete from bans where ip='$ip'";
    $result = $db->query($sql);
    return DB::isError($result) ? false : true;
}

/**
* Determine of access is from a banned IP
*
* @return boolean Whether request is from banned IP
*/
function isBanned() {
    global $db;
    $sql = "select ip from bans where '${_SERVER["REMOTE_ADDR"]}' like ip";
    $result = $db->getOne($sql);
    return $result ? $result : false;
}

/**
* Stupid little function to auto-generate a menu of categories. This
* involved the page generation class instance $page, and that the template
* contains a placeholder named "CATEGORIES". If you don't want the menu
* of categories auto-generated, don't call this function and instead write
* out your own menu manually in site.tmpl
*
* @return void
*/
function setCategories() {
	global $db,$page;
	$sql = "select * from categories order by category";
	$result = $db->query($sql);
	$menu = "";
	while($row = $result->fetchRow()) {
		$menu .= sprintf('<a href="%s/%s" title="%s">%s</a>%s',
							ROOT,
							$row["abbr"],
							$row["description"],
							$row["category"],
							"\n");
	}
	$page->set("CATEGORIES",$menu);
}

/**
* Create or update a category record. If $id is provided, it's an
* update ($abbr ignored) or if $abbr is provide, an insert.
*
* @param	int		$id 			Record ID of category to update
* @param	string	$category		Name of category
* @param	string	$description	Description of category
* @param	string	$sort			Sort method for category
* @param	string	$abbr			Abbreviation/URL for category
* @return	boolean	Status
*/
function updateCategories($id,$category,$description,$sort,$abbr=false) {

	global $db;

	if(!$category || !$description || !$sort) {
		return $false;
	}
	if(!$id && !$abbr) {
		return false;
	}
	$category = $db->escapeSimple($category);
	$description = $db->escapeSimple($description);
	$abbr = $db->escapeSimple($abbr);
	$sort = $db->escapeSimple($sort);

	if(is_numeric($id)) {
		$sql = "update categories set category='$category',description='$description',sort='$sort' where id='$id'";
	} else {
		$sql = "insert into categories(category,abbr,description,sort) values ('$category','$abbr','$description','$sort')";
	}
    $result = $db->query($sql);
    return DB::isError($result) ? false : true;
}


/**
* Fetch next id for new image request
*
* This one is problematic. Databases handle this differently, and some (sqlite)
* don't have a real sequence type so you can't make a call to get the next one
* and be assured it's not going to be given out while you are working on it.
* 
* In other words: RACE CONDITION for image posts (where this is used). The
* problem being that before we can process the image, you need an id for
* naming purposes, but you don't want to insert the sql record until you're
* sure the image processed OK.
*
* I'll have to re-visit this at some point. It's not *hard*, per se, I just
* want to do it elegantly.
*
* @param none
* @return int The next available rowid for the images table
*
*/
function getNextId() {
	global $db;
	$sql = "select max(id) from images";
	$result = $db->getOne($sql);
    return DB::isError($result) ? false : $result + 1;
}

/**
* Insert a new image record
*
* @param	int		$id			Record ID
* @param	int		$parent		Record ID of parent, if a reply
* @param	string	$name		Name of poster
* @param	string	$tripcode	Calculated tripcode of poster
* @param	string	$title		Title for image
* @param	string	$comment	Comment for post
* @param	string	$categories	Category string
* @param	string	$keywords	Keywords
* @param	string	$file		File name for image
* @param	string	$type		File (image) type
* @param	boolean	$animated	Whether is animated or not
* @param	int		$size		Size of file, in bytes
* @param	string	$resolution	Resolution of image, as "123 x 456"
* @return	boolean	Status
*/
function post($id,$parent,$name,$tripcode,$title,$comment,$categories,$keywords,$file=false,$type=false,$animated=false,$size=false,$resolution=false) {

	global $db;
    $name = $db->escapeSimple($name);
    $keywords = $db->escapeSimple($keywords);
    $title = $db->escapeSimple($title);
    $comment = $db->escapeSimple($comment);
	$parent_insert = $parent ? "'$parent'" : "null";
	$date = date("U");
    $animated = $animated==1 ? 1 : 0;
    $size = $size ? $size : 0;

	$sql = "insert into images (id,parent,name,tripcode,title,file,type,animated,size,resolution,keywords,category,date,ip,mtime,comment,sort) values (%s,%s,'%s','%s','%s','%s','%s','%s','%s','%s','%s','%s','%s','%s','%s','%s','%s')";
	$sql = sprintf($sql,$id,$parent_insert,$name,$tripcode,$title,$file,$type,$animated,$size,$resolution,$keywords,$categories,$date,$_SERVER["REMOTE_ADDR"],$date,$comment,$id);

    $result = $db->query($sql);
    return DB::isError($result) ? false : true;
}

/**
* Get a count of how many image threads match a set of criteria.
* This is used to draw the pager, and in many ways mirrors
* getImages(), except it only returns a count, rather than the full result
* set. Perhaps they should be merged.
*
* @param	string	$which
*		Query type. This can be "search" (in which case $search) is required,
*		"keyword" (again $search required), a category, or false
*
* @param	string	$search
*		This contains the query terms for search and keyword types
*
* @param	string	$fields
*		For search types, this contains the columns to search for $search
*
* @return	int The number of threads that match the above
*/
function getCount($which=false,$search=false,$fields=false,$categories=false) {
	global $db;
	if($which == "search" && $search && is_array($fields)) {
		$sql = "select count(*) from images where";
		$search = $db->escapeSimple($search);
		$clauses = array();
		foreach($fields as $field) {
			array_push($clauses,"$field like '%$search%'");
		}
		$sql .= " (" . implode(" or ",$clauses) . ")";
	} else if($which == "keyword" && $search) {
		$search = $db->escapeSimple($search);
		$sql = "select count(*) from images where keywords like '%$search%'";
	} else if($which) {
		$sql = "select count(*) from images where parent is null and category like '%/$which/%'";
	} else {
		$sql = "select count(*) from images where parent is null";
	}
    if($which =="search" || $which == "keyword") {
        $cclauses = array();
        if($categories) {
            $cats = explode(",",$categories);
            foreach($cats as $c) {
                $cclauses[] = "category like '%/" . $db->escapeSimple($c) . "/%'";
            }
        }
        if(count($cclauses)>0) {
            $sql .= " and (" . implode(" or ",$cclauses) . ")";
        }
    }
	$result = $db->getOne($sql);
    return DB::isError($result) ? false : $result;
}

/**
* Get image threads
* This only gets thread starters. A separate call to getComments() is required
* to get the replies.
*
* @param	string	$which
*		Query type. This is the category as determined by the image.php
*		"search" ($search, $fields required) - generalized searches
*		"keyword" (again $search required), a category, or false
*
* @param	int		$offset
*		Offset (paging) for the query
*
* @param	string	$search
*		This contains the query terms for search and keyword types
*
* @param	string	$sort
*		Sorting criteria. This is not simply read out of the categories table
*		because it may differ depending on the meta queries such as search
*		and keywords
*
* @param	string	$fields
*		For search types, this contains the columns to search for $search
*
* @return	int Indexed array of associative arrays (column=>value)
*/
function getImages($which,$offset,$search=false,$sort="bump",$fields=false,$categories=false) {
	global $db;

    if($which == "search" || $which == "keyword") {
        $cclauses = array();
        if($categories) {
            $cats = explode(",",$categories);
            foreach($cats as $c) {
                $cclauses[] = "category like '%/" . $db->escapeSimple($c) . "/%'";
            }
        }
    }
	if($which == "All") {
		$sql = "select * from images where parent is null order by mtime desc,id desc " .
                limitOffset(LIMIT,$offset);
	} else if ($which == "Stats") {
		$sql = "select * from images order by views desc,id desc limit " . LIMIT;
	} else if (is_numeric($which)) {
		$sql = "select * from images where images.id='$which'";
	} else if ($which=="search" && $search && is_array($fields)) {
		$search = $db->escapeSimple($search);
		$clauses = array();
		foreach($fields as $field) {
            array_push($clauses,"$field like '%$search%'");
		}
        if(count($clauses)<1) {
            return false;
        }
		$sql = "select * from images where (" . implode(" or ",$clauses) . ")";
        if(count($cclauses)>0) {
            $sql .= " and (" . implode(" or ",$cclauses) . ")";
        }
        $sql .= " order by id asc " . limitOffset(LIMIT,$offset);
	} else if ($which == "keyword" && $search) {
		$search = $db->escapeSimple($search);
		$sql = "select * from images where keywords like '%$search%'";
        if(count($cclauses)>0) {
            $sql .= " and (" . implode(" or ",$cclauses) . ")";
        }
        $sql.= " order by sort asc " . limitOffset(LIMIT,$offset);
	} else {
		$sql = "select * from images where parent is null and category like '%/$which/%' ";
		switch($sort) {
		case "asc":
			$sql .= " order by sort asc";
			break;
		case "desc":
			$sql .= " order by sort desc";
			break;
		default:
			$sql .= "order by mtime desc,sort desc";
			break;
		}

        $sql .= " " . limitOffset(LIMIT,$offset);
	}
	$result = $db->getAll($sql);
    return DB::isError($result) ? false : $result;
}

/**
* Fetch comments, given parent specifiers
*
* @param array $which Array of parent IDs for comments to fetch
* @return array Associative array, keyed by parent id of associative arrays
*/
function getComments($which) {	
	global $db;
	if(!is_array($which) || count($which)<1) {
		return;
	}
	$which = "'" . implode("','",$which) . "'";
	$sql = "select * from images where parent in ($which) order by sort asc";
	$result = $db->getAll($sql);
	if(DB::isError($result)) {
		return false;
	}
	
	$comment_hash = array();
    foreach($result as $row) {
		$comment_hash[$row["parent"]][] = $row;
	}
	return $comment_hash;
}

/**
* Return count of comments for given parent id
*
* @param int $id Parent id
* @return int Count of comments in that thread
*/
function getNumComments($id) {
	global $db;
	$sql = "select count(*) from images where parent='$id'";
	$result = $db->getOne($sql);
    return DB::isError($result) ? false : $result;
}

/**
* Bump a thread by updating mtime field for parent
*
* @return boolean status
*/
function bump($id) {
	global $db;
	$sql = "update images set mtime='" . date("U") . "' where id='$id'";
    $result = $db->query($sql);
    return DB::isError($result) ? false : $result;
}

/**
* Return a single image record
*
* @param int $id images.id for record
* @return array Associative array for the record
*/
function getImage($which) {
	global $db;
	if(!is_numeric($which)) {
		return false;
	}
	$sql = "select * from images where id='$which'";
	$result = $db->getRow($sql);
    return DB::isError($result) ? false : $result;
}

/**
* This handles delivering full images, given the filename. This works this way
* to maintain the integrity of the views counter. If the views call wasn't used
* the direct URL could be passed around, circumventing views counting. Doing it
* this way forces view counting and image viewing to be tied.
*
* Of course, a determined individual could reverse engineer the proper URL for
* the image file, but this isn't for that - it's only for inadvertent circum-
* vention. If you really wanted to stop that too, you'd move the images dir
* out of the web document space.
*
* Note: the usage of this function has serious performance issues so you
* should consider dropping the use of the views counter if you have high-
* performance needs.
*/
function viewImage($file) {
	global $db;
	$file = $db->escapeSimple($file);
	$sql = "select * from images where file='$file'";
	$image = $db->getAll($sql);
	if(!$image || count($image)!=1) {
		return false;
	}
    $image = $image[0];
	$sql = "update images set views=views+1 where id='${image["id"]}'";
	$result = $db->query($sql);
	if(DB::isError($result)) {
		return false;
	}
    $file = getImageUrl($image["id"],$image["file"],false,true);
    $info = getImageSize($file);
    header("Content-Type: " . $info["mime"]);
    readfile(getImageUrl($image["id"],$image["file"],false,true));
	exit;
}

/**
* Fetch results appropriate for RSS output
*
* @param string $category The category (will use "All" if not provided)
* @return array Indexed array of associative arrays
*/
function getRSS($category) {
	global $db;

	$sql = "select * from images left join (select parent,count(id) as replies,count(file) as imgreplies from images where parent is not null group by parent) as replies on (images.id=replies.parent) where images.parent is null";
	if($category) {
		$sql .= " and category like '%/${category}/%'";
	} 
	$sql .= " order by id desc limit " . LIMIT;
	$result = $db->getAll($sql);
    return DB::isError($result) ? false : $result;
}

function limitOffset($limit,$offset) {
    global $db;
    switch($db->phptype) {
        case "mysql":
            return "limit $offset,$limit";
            break;
        default:
            return "limit $limit offset $offset";
            break;
    }
}

/**
* Update an image record
*
* @param int $id images.id of the record to update
* @param string $title The new title
* @param string $categories The new categories value
* @param string $keywords The new keywords value
* @param string $comment The new comment value
* @return boolean Status
*/
function modifyImage($id,$title,$categories,$keywords,$comment) {
	global $db;

	if(!$id || !$categories) {
		return false;
	}
	$title = $title ? $db->escapeSimple($title) : "";
	$keywords = $keywords ? $db->escapeSimple($keywords) : "";
	$sql = "update images set title='$title',category='$categories',keywords='$keywords'";

	if($comment !== false) {
		$comment = $comment ? $db->escapeSimple($comment) : '';
		$sql .= ",comment='$comment'";
	}
	
	$sql .= " where id='$id'";

    $result = $db->query($sql);
    return DB::isError($sql) ? false : true;
}

/**
* Update a comment
* Like modifyImage(), but while that handled parents, this handles replies
*
* @param int $id images.id for record to update
* @param string $comment The new comment
* @return boolean status
*/
function updateComment($id,$comment) {
	global $db;

	if(!$id) {
		return false;
	}
	$comment = $comment ? $db->escapeSimple($comment) : '';
	$sql = "update images set comment='$comment' where id='$id'";
    $result = $db->query($sql);
    return DB::isError($sql) ? false : true;
}

/**
* Delete an image. This deletes an image record from the database
* after deleting the image file.
*
* If the parent of thread is to be deleted then the record isn't actually
* deleted. Rather, it's replaced with a "deleted" notification image. This
* is to preserve comments that may have been in reply to it, and prevent those
* from being deleted arbitrarily by the original thread starter.
*
* @param id $which images.id for the record to delete
* @return boolean Status
*/
function deleteImage($which) {
	global $db;
	if(!is_numeric($which)) {
		return false;
	}
	$image = getImage($which);
	if($image === false) {
		return false;
	}

	if($image["file"] && $image["file"] != DELETED_IMAGE) {
		// Start with the thumbnail - least important
		$file = sprintf("%s/%02d/%s",THUMB_DIR,floor($image["id"]/1000),$image["file"]);
		if(!@unlink($file)) {
			return false;
		} 

		// Next, the image
		$file = sprintf("%s/%02d/%s",IMAGE_DIR,floor($image["id"]/1000),$image["file"]);
		if(!@unlink($file)) {
			return false;
		} 
	}
	
	// Now the database. If #comments in thread>0, then don't delete the entry.
	// Instead, replace file value with "deleted" image
	if(DB::isError($db->query("begin"))) {
		return false;
	}
	$sql = "select count(*) from images where parent='$which'";
    $result = $db->getOne($sql);
    if(DB::isError($result)) {
        $db->query("rollback");
        return false;
    }
	if($result==0) {
		$sql = "delete from images where id='$which'";
	} else {
		$sql = "update images set file='" . DELETED_IMAGE . "' where id='$which'";
	}
	
    $result = $db->query($sql);
    if(DB::isError($result)) {
        $db->query("rollback");
        return false;
	} 

	// OK, made it...
    $db->query("commit");
	return true;
}

/**
* Check whether request comes from a flooding IP
*
* @return mixed Flooding (returns number) or not (false)
*/
function isFlooding() {
	global $db;
	$window = date("U") - FEATURE_THROTTLE_PERIOD * 60;
	$subnet = ereg_replace("\.[^\.]+$",'',$_SERVER["REMOTE_ADDR"]);
	$sql = "select count(*) from images where ip like '$subnet%' and date>'$window'";
	$num = $db->getOne($sql);
	if($num>=FEATURE_THROTTLE_NUMBER) {
		return $num;
	} else {
		return false;
	}
}

/**
* Ban an IP
* @param string $ip The ip address or glob (ie '127.0.0.*')
* @param string $reason Reason for ban
* @param string $tripcode Tripcode for moderator making the ban
* @return boolean Status
*/
function addBan($ip,$reason,$tripcode) {
    global $db;
    $reason = $reason ? $db->escapeSimple($reason) : "";
    $date = date("U");
    $sql = "insert into bans(ip,date,tripcode,reason) values ('$ip','$date','$tripcode','$reason')";
    $result = $db->query($sql);
    return DB::isError($result) ? false : true;
}

/**
* Ugly nasty queries for fetching data for stats page
*
* @return array The data, structured as:
*   $array[whichstats][category] = count
*/
function getStats() {
	global $db;
	$return = array();
	// I really should find better ways to doing these queries

	$queries = array(
		"total_posts"	=> "select c.abbr as abbr,count(*) as num from categories c join v_categorized v on (v.category like '%/' || c.abbr || '/%') group by c.abbr",
		"num_threads"	=> "select c.abbr as abbr,count(*) as num from categories c join v_categorized v on (v.category like '%/' || c.abbr || '/%') and v.parent is null  and v.file!='' group by c.abbr",
		"num_images"	=> "select c.abbr as abbr,count(*) as num from categories c join v_categorized v on v.file!='' and v.category like '%/' || c.abbr || '/%' group by c.abbr",
		"total_size"	=> "select c.abbr as abbr,sum(size) as num from categories c join v_categorized v on v.file!='' and v.category like '%/' || c.abbr || '/%' group by c.abbr",
		"num_views"	=> "select c.abbr as abbr,sum(views) as num from categories c join v_categorized v on v.file!='' and v.category like '%/' || c.abbr || '/%' group by c.abbr",
	);

	foreach($queries as $label=>$sql) {

        $result = $db->getAll($sql);
        if(DB::isError($result)) {
            die($result->getDebugInfo());
			return false;
		}
		$return[$label] = array();
        foreach($result as $row) {
			$return[$label][$row["abbr"]] = $row["num"];
		}
	}

	return $return;
}

/**
* Keyword counter, to show which keywords are popular
*
* @param int $num Top $num popular keywords returned
* @return mixed Array of $word=>$count, or false
*/
function getPopular($num) {
	global $db;
	$sql = "select keywords from images where keywords != ''";
	$result = $db->getCol($sql);
    if(DB::isError($result)) {  
        return false;
    }
    $usage = array();
    foreach($result as $x) {
        $words = explode(" ",$x);
        foreach($words as $word) {
            $usage[$word]++;
        }
    }
    arsort($usage);
    array_splice($usage,$num,count($usage));
    return $usage;
}

/**
* Attempt to re-order a image.
* This is complicated by the fact that an image may appear in a number
* of different contexts, depending on which category or categories it's
* in, how those categories are sorted, or if it's in a keyword view
*
* The general strategy: look up the sort value (by default sort=id) for
* the parent it should appear *after* and the sort value of the image that
* follows it, then give this image a sort value between them.
*
* @param int $id images.id for the image to reorder
* @param int $target images.id for the image that should appear "before"
* @param string $sort The sort order we are expecting
* @return boolean Status
*/
function reorder($id,$target,$sort) {

    global $db;

    // First, get the two image sort IDs that this image will fall between
    if(DB::isError($db->query("begin"))) {
        return false;
    }
    $sql = "select sort from images where sort ";
    $sql .= $sort == "asc" ? ">=" : "<=";
    $sql .= " $target order by sort $sort limit 2";
    $result = $db->getCol($sql);

    // Now, split the different, or add 0.5 if at the edges
    if(count($result)==1) {
        $new_sort_value = $sort == "asc" ? $target+0.5 : $target - 0.5;
    } else {
        $new_sort_value = ($result[0]+$result[1])/2;
    }
    $sql = "update images set sort='$new_sort_value' where id='$id'";
    $result = $db->query($sql);
    if(DB::isError($result)) {
        $db->query("rollback");
        return false;
    } else {
        $db->query("commit");
        return true;
    }
}

/**
* Generate a image url (or file path)  from parts known about it.
* Doing this because it is not stored in full in the database, and there is
* some special processing to determine where it is. By default here,
* images are stored in subdirectories named with two digits ($id/1000) so
* that a directory may only have 1000 images.
*
* @param int $id images.id
* @param string $name The image file name
* @param boolean $thumb Whether it's the thumbnail url we want
* @param boolean $filepath Whether it is the file path we wan
* @return string The url/file path
*/
function getImageUrl($id,$name,$thumb=false,$filepath=false) {
	if ($filepath) {
		$dir = $thumb ? THUMB_DIR : IMAGE_DIR;
	} else {
		$dir = $thumb ? THUMB_URL : IMAGE_URL;
	}
	if($name==DELETED_IMAGE) {
		return "img/deleted.gif";
	} else if (!$name) {
		return sprintf("%s/%02d",$dir,floor($id/1000));
	} else {
		return sprintf("%s/%02d/%s",$dir,floor($id/1000),$name);
	}
}

/**
* Delete stuff in a directory (from zip processing)
*
* @param string $dir The directory to clean up
* @return void
*/
function cleanUpDir($dir,$rmdir=true) {
	$d = opendir($dir);
	while(false != ($filename = readdir($d))) {
		if($filename != "." and $filename != "..") {
			@unlink("$dir/$filename");
		}
	}
	if($rmdir) {
        @rmdir($dir);
    }
}

/**
* Generate a verification code.
* Note that it's tied to the request's IP address and User-Agent string!
*
* @param string $s Seed
* @return string The verification code
*/
function verification($s) {
    $code = tripcode($s . $_SERVER["REMOTE_ADDR"] . $_SERVER["HTTP_USER_AGENT"] . TRIPCODE_SECRET);
    $code = strtoupper(substr($code,0,6));
	$code = ereg_replace("[^A-z]","",$code);
    return $code;
}

/**
* Calculate tripcode.
* If you are using PHP5, you should uncomment the first part, it is better
* You may also want to change from md5() to sha() if you want a bit more
* robustness in the cryptography. Or increase the length of the substring
*
* @param string $s The user supplied tripcode string
* @return string The calculated tripcode
*/
function tripcode($s) {
	//----- With PHP5,  this is all you would have to do
	#$x = base64_encode(md5($s . TRIPCODE_SECRET,true));
	#return substr($x,0,10);
	//----- END PHP5

	$x = md5($s . TRIPCODE_SECRET);
	for($i=0;$i<32;$i+=2) {
		$a = hexdec(substr($x,$i,2));
		$bin .= chr($a);
	}
	$x = base64_encode($bin);
	return substr($x,0,10);
}
?>
