<?php
/* vim: set et ts=4 sw=4 sts=4 ai: */
// +----------------------------------------------------------------------+
// | Image post handler                                                   |
// +----------------------------------------------------------------------+
// | @version $Revision: 1.20 $
// +----------------------------------------------------------------------+

//----- Sanity check - this file shouldn't be referenced directly
//		and htaccess should be configured to disallow access to this dir
if(!$page) { exit; }


//----- If $verify sent as $_GET, then it's a preverify request so we
//      just need to send the confirmation
if($_GET["verify"] && FEATURE_VERIFY) {
    $should_be = strtolower(verification($_SESSION["seed"]));
    $was = strtolower($_GET["verify"]); 
    echo ($should_be == $was) ? "true" : "false";
    exit;
}

if($_POST) {
    $errors = array();


    //----- Restricted posting? Allow comments, but any image upload
    //      and thread starting attempts should be aborted
    if(FEATURE_RESTRICT_POSTING && ($_POST["file"] || !$_POST["parent"])) {
        $tripcodes = explode(",",FEATURE_RESTRICT_POSTING);
        if(!in_array(tripcode($_POST["tripcode"]),$tripcodes)) {
            $page->fail($ERRORS["restricted"]);
        }
    }

    //----- Check throttling
    if(FEATURE_THROTTLE===true) { 
        $tripcodes = explode(",",FEATURE_THROTTLE_EXCEPTIONS);
        if(!in_array(tripcode($_POST["tripcode"]),$tripcodes)) {
            if ($num=isFlooding()) {
                $page->fail($STRINGS["flood"]);
            }
        }
    }

    //----- Check verification code
    if(FEATURE_VERIFY) {
        $should_be = verification($_SESSION["seed"]);
        $was = $_POST["verify"];
        if(strtolower($should_be) != strtolower($was)) {
            array_push($errors,"Verification mismatch");
        }
    }

    //----- Was parent passed? If so, we're doing a reply
    $parent = is_numeric($_POST["parent"]) ? $_POST["parent"] : false;

    // Name is required
    if($_POST["name"]) {
        $name = $_POST["name"];
        setcookie("name",$_POST["name"],COOKIE_EXPIRES);
    } else {
        array_push($errors,$ERRORS["namerequired"]);
    }

    // Categories - required for thread starters
    $categories = false;
    if(!$parent) {
        if(FEATURE_MULTIPLE_CATEGORY===true && is_array($_POST["categories"]) && count($_POST["categories"])>0) {
            $categories = "/" . implode("/",$_POST["categories"]) . "/";
        } else if($_POST["categories"]) {
            $categories = "/" . $_POST["categories"] . "/";
        }

        if(!$categories) {
            array_push($errors,$ERRORS["categoryrequired"]);
        }
    }

    // Title is not required
    $title = $_POST["title"];

    // Tripcode is not required
    $tripcode = $_POST["tripcode"] ? tripcode($_POST["tripcode"]) : "";
    setcookie("tripcode",$_POST["tripcode"],COOKIE_EXPIRES);

    // Keywords - not required
    $keywords = $_POST["keywords"];

    $image_posted = false;
    // If URL fetching, needs to be valid.
    if ($_POST["url"]) {
        if(!ereg("^http://",$_POST["url"]) && !ereg("^ftp://",$_POST["url"])) {
            array_push($errors,$ERRORS["urlformat"]);
        } else {
            $referrer = ereg_replace("(^.+//[^/]+).+","\\1/",$_POST["url"]);
            $image_posted = true;
        }
    } else if (is_uploaded_file($_FILES['file']['tmp_name'][0])) {
        $image_posted = true;
        // Was it a zip
        $cmd = "file " . $_FILES['file']['tmp_name'][0];
        exec($cmd,$out);
        $zip_posted = strstr($out[0],"Zip archive data") ? true : false;

        // Only allowed tripcodes can upload zip
        if($zip_posted) {
            $tripcodes = explode(",",FEATURE_ZIP_UPLOADERS);
            if(!in_array(tripcode($_POST["tripcode"]),$tripcodes)) {
                $page->fail($ERRORS["restricted"]);
            }
        }
    }

    // Quit now if errors ...
    if(count($errors)>0) {
        echo "<p>${ERRORS["errorsoccurred"]}</p>\n";
        echo implode("</li>\n<li>",$errors);
        echo "</li></ul>\n";
        $page->fail('');
    }

    $comment = $_POST["comment"][0];

    // Either an image or a comment is required - empty posts are rejected
    if(!$image_posted && !$comment) {
        $page->fail($ERRORS["emptyreject"]);
    }

    // Get an id for this record
    $date = date("U");
        
    // May alter $parent value if multi-file posting, so store here
    // for use later
    $original_parent = $parent;

    //////////////////////////////////////////////////////////////////////////
    // Here be dragons
    //
    // Images can be posted in many ways. A URL to an image can be provided,
    // a zip file containg images can be uploaded, or direct HTTP-based
    // uploads may be posted (in which case there may be multiple).
    //
    // Each of the 3 are considered mutually exclusive - if a URL is provided
    // then the file uploads are ignored; if a zip file is found as the first
    // file uploaded, no further processing of other files is performed. The 
    // standard file upload is the fall back.
    //
    // This next block tries to figure all this out, and create a $files array
    // that contains information on the target file name, the path to the file
    // as it resides on the file system, and a comment if/when applicable.
    if($image_posted) {

        // If it was a URL fetch, fetch it now
        if($_POST["url"]) {
            $filename = basename($_POST["url"]);
            $filepath = TMP_DIR . "/$filename";

            // Just one file
            $files = array(array($filename,$filepath,$comment));

            unset($out);
            // only catch stderr
            $cmd = sprintf(WGET,WGET_USER_AGENT,$referrer,$filepath,$_POST["url"]);
            exec($cmd,$out);
            if(count($out)>0) {
                $page->fail($ERRORS["fetchfail"]);
            }

        // If it was a zip?
        } else if ($zip_posted) {
            // Make a tmp dir
            $tmpdir = uniqid("/tmp/",true);
            if(!@mkdir($tmpdir,0700)) {
                $page->fail($ERRORS["mkdirfail"]);
            }
            unset($out);
            if($zip_posted) {
                $cmd = "unzip -q -d $tmpdir -j " . $_FILES['file']['tmp_name'][0];
            }
            // Catch stderr
            $cmd .= " 2>&1";
            exec($cmd,$out);
            if(count($out)>0) {
                var_dump($out);
                cleanUpDir($tmpdir);
                $page->fail("Zip processing failed");
            }

            // In PHP5, scandir could be used
            $files = array();
            $d  = opendir($tmpdir);
            while (false !== ($filename = readdir($d))) {
                if($filename != "." && $filename != "..") {
                    // Spaces in file names cause problems
                    $newname = str_replace(" ","_",$filename);
                    @rename("$tmpdir/$filename","$tmpdir/$newname");
                    array_push($files,array($newname,"$tmpdir/$newname",$comment));
                    // The comment is only applied to the first
                    $comment = null;
                }
            }
            sort($files);

        } else {

        // Else... a standard upload
        // This gets tricky. Multiple files can be posted, and they come in
        // "slots" and there is nothing to prevent a user from posting
        // discontinuously. And each may or may not have a comment attached.
        //
        // So, the strategy here is to only process multiple files if they are
        // "continuous"- they shouldn't invoke the multiple-file stuff if they
        // aren't really going to post multiple files (ie multiple comment-only
        // posts are ignored.

            if(is_numeric(FEATURE_MULTIPLE_UPLOAD)) {
                $num_allowed = count($_FILES['file']['name'])>FEATURE_MULTIPLE_UPLOAD ?
                                    FEATURE_MULTIPLE_UPLOAD :
                                    count($_FILES['file']['name']);
            } else {
                $num_allowed = 1;
            }
            $files = array();
            for($i=0; $i<$num_allowed;$i++) {
                $filename = $_FILES['file']['name'][$i];
                $filepath = $_FILES['file']['tmp_name'][$i];

                // Just one file. Break out on first encounter empty
                if($filename && $filepath) {
                    $files[] = array($filename,$filepath,$_POST["comment"][$i]);
                } else {
                    break;
                }

            }
            // Force posting into 1 thread, if new thread
            if(!$_POST["parent"]) {
                $_POST["thread"] = 1;
            }

        }

        include(INCLUDES_DIR . "/process_image.php");

        $i=0;
        foreach($files as $file) {
            $id = getNextId();
            $image = processImage($file[1],$file[0],$id,$parent);

            if($image[0]!==true) {
                if($tmpdir) {
                    cleanUpDir($tmpdir);
                }
                $page->fail($image[1]);
            }

            $image = $image[1];
            $comment = $file[2];

            if($parent) {
                // No title,keyword,categories
                $result = post($id,$parent,$name,$tripcode,false,$comment,false,false,$image["name"],$image["type"],$image["animated"],$image["size"],"${image["width"]} x ${image["height"]}");
            } else {
                $result = post($id,$parent,$name,$tripcode,$title,$comment,$categories,$keywords,$image["name"],$image["type"],$image["animated"],$image["size"],"${image["width"]} x ${image["height"]}");
            }

            // If multi-file post, and posting to a thread, then set $parent
            // for subsequent entries.
            if($i==0 && $_POST["thread"]) {
                $parent = $id;
            }

            // Unset comment. For single file posts, we already took care oof
            // it. For multi-file posts, it should've only applied to the first.
            $comment = false;

            $i++;
        }

        if($tmpdir) {
            cleanUpDir($tmpdir);
        }

    } else {
        // Just text
        $id = getNextId();
        if($parent) {
            $result = post($id,$parent,$name,$tripcode,false,$comment,false,false);
        } else {
            $result = post($id,false,$name,$tripcode,$title,$comment,$categories,$keywords);
        }
        $i=1;
    }

    if(!$result) {
        $page->fail($ERRORS["queryfail"]);
    }

    //----- Bump? If FEATURE_SAGE, then only bump if #comments<FEATURE_SAGE
    if(strtolower($_POST["bump"]) == "on") {
        $bump = true;
        if(is_numeric(FEATURE_SAGE)) {
            $num = getNumComments($original_parent);
            // Since multiple files could've been uploaded, have to calculate
            // what it was *prior* to this upload
            if(($num-$i)>=FEATURE_SAGE) {
                $bump = false;
            }
        }
        if($bump) {
            bump($original_parent);
        }
    }

    echo $STRINGS["postthanks"];
    echo JS_AUTO_CLOSE;

    $seed = mt_rand();
    $_SESSION["seed"] = $seed;

} else {

    $seed = mt_rand();
    $_SESSION["seed"] = $seed;

    // Find out where we got popped from...
    $referrer = basename($_SERVER["HTTP_REFERER"]);

    // Set up template, and variable filler
    $tmpl = new HTML_Template_IT(TEMPLATE_DIR);
    $tmpl->loadTemplateFile("post.tmpl");
    $vars = array();

    // Is this a thread-starter or a reply?
    $id = $_REQUEST["id"];
    if(is_numeric($id)) {
        
        // REPLY

        $image = getImage($id);
        // Can't comment on comments ;)
        if($image === false || $image["parent"]) {
            $page->fail($ERRORS["queryfail"]);
        }

        $title = $image["title"] ? $image["title"] : $STRINGS["untitled"];
        $title = htmlspecialchars(stripslashes($title));
        $vars["title"] = sprintf($STRINGS["commenttitle"],$title,$image["name"]);
        $vars["id"] = $image["id"];
        if($image["file"]) {
            $vars["thumb_url"] = getImageUrl($id,$image["file"],true);
            $vars["thumb_title"] = $vars["title"];
        }

        $tmpl->touchBlock("bump");

    } else {

        // THREAD-STARTER

        // Categories...
        $categories = getCategories();
        foreach($categories as $row) {
            if ($row["abbr"] == $referrer) {
                $row["checked"] = 'checked="true"';
            }
            $tmpl->setCurrentBlock("category");
            $tmpl->setVariable(array(
                "category_name"     => FEATURE_MULTIPLE_CATEGORY ? "categories[]" : "categories",
                "category_type"     => FEATURE_MULTIPLE_CATEGORY ? "checkbox" : "radio",
                "category_value"    => $row["abbr"],
                "checked"           => $row["checked"],
                "label"             => $row["category"],
            ));
            $tmpl->parseCurrentBlock();
        }
        $tmpl->touchBlock("categories");
        $tmpl->touchBlock("title");
        $tmpl->touchBlock("keywords");
        $tmpl->touchBlock("threaded");

        $vars["title"] = $STRINGS["posttitle"];

    }

    // If FEATURE_RESTRICT_POSTING, then don't show the file upload input
    // on reply forms unless allowed to
    $show_file_upload = false;
    if(!is_numeric($id)) {
        $show_file_upload = true;
    } else {
        if(!FEATURE_RESTRICT_POSTING) {
            $show_file_upload = true;
        } else {
            $tripcodes = explode(",",FEATURE_RESTRICT_POSTING);
            if(in_array(tripcode($_COOKIE["tripcode"]),$tripcodes)) {
                $show_file_upload = true;
            }
        }
    } 

    if($show_file_upload) {
        $tmpl->touchBlock("file_upload");
        $vars["maximum_size"] = MAXIMUM_SIZE;
        $vars["max_bytes"] = round(MAXIMUM_SIZE/1024/1024);

        if(FEATURE_FETCH===true) {
            $tmpl->touchBlock("url_fetch");
        }
    }
        
    // Both thread-starters and replies

    if(FEATURE_VERIFY===true) {
        $vars["seed"] = $seed;
        $tmpl->touchBlock("verify_help");
    }

    $vars["cookie_name"] = $_COOKIE["name"];
    $vars["max_uploads"] = is_numeric(FEATURE_MULTIPLE_UPLOAD) ? FEATURE_MULTIPLE_UPLOAD : 1;
    $tmpl->setVariable($vars);
    $tmpl->show();
}
?>
